/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CARDDATA_H
#define CARDDATA_H

#include <qptrlist.h>
#include <qstring.h>
#include <stdio.h>
#include <qdatetime.h>

#include "wcard.h"
#include "eperiod.h"
#include "consts.h"

class CalendarCardWidget;


/**
Class stores all information about one day - date and lists of periodic and individual events.<br>
This class is sometimes connected to CalendarCardWidget object, what means, this day is currently visible and has his own place on the screen.

@brief Day information class
@author Adam Skowronski
*/
class CalendarCardData{ 
public:
    CalendarCardData();

    ~CalendarCardData();
    CalendarCardData(QDate date, QWidget * parent);
 
	//returns text to display
    QString getIndividualEventText();
    QString getPeriodicEventText();
 
	int setIndividualEvent(QString tekst, bool bIsPast);
	
    int addPeriodicEvent(CalendarEventPeriodic *event);
	int addIndividualEvent(CalendarEvent *event);
	
	int resetCard();
    QDate getDate();

    int removePeriodicEvents();
    CalEnums::PIXMAP_STATES evaluateState();

    /*!
        \fn CalendarCardData::getState()
		Standard getter<br>Returns the state of CalendarCardData \sa state
     */
    CalEnums::PIXMAP_STATES getState()
    {
        return state;
    }
 
	
	QWidget*  parent; /**< Pointer to an application window. Used for call some application method */ 
  	QPtrList<CalendarEventPeriodic> PeriodicEvents; /**< list of periodic events. It stores a list of events taken from global list of periodic events from CalendarWidget object. */
	CalendarEvent* IndividualEvent;/**< pointer to individual event. <br>It can be only one individual event per object */
    CalEnums::PIXMAP_STATES state;/**< State of object. <br>See CalEnums::PIXMAP_STATES for possible values and detailed description */
	QDate date;    /**< A date connected to object */
	CalendarCardWidget * pWidget;/**< A pointer to widget object which data from CalendarCardData object is displayed in. <br>If object shouldn't be visible value of this pointer should be NULL */
	bool isFirstOfMonth;/**< A boolean value which is used by application window to decide if it is a first day of mont and to draw the name of month below the card */
 
	
};

#endif
