/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "fpast.h"
#include <qobject.h>
#include <qmessagebox.h>
#include "consts.h"

/*!
	\fn FilePast::FilePast(QPtrList<CalendarEventPast>* l)
	Constructor of a class
	\param l pointer to global list of past events
*/
FilePast::FilePast(QPtrList<CalendarEventPast>* l)
{
	list=l;
}

/*!
	\fn FilePast::~FilePast()
	Default destructor
*/
FilePast::~FilePast()
{
}

#include <iostream.h>

/*!
    \fn FilePast::createPastXML()
	Methods creates xml string, which will be used to save in a file, using list of past events
	\return String to save in a file
*/
QString FilePast::createPastXML()
{
	QString text_xml;
	//create doc with <PastEvents>
	text_xml+="<PastEvents>\n";
	//insert <head> section
	text_xml+="\t<header>\n";
		//insert <description>
	text_xml+="\t\t<description>";
	text_xml+=QObject::tr("File with past events");
	text_xml+="</description>\n";
	text_xml+="<SavingDate>"+QDate::currentDate().toString("yyyyMMdd")+"</SavingDate>\n";
	text_xml+="\t</header>\n";
	//insert <data> section
	text_xml+="\t<data>\n";
		//for each past event
		for(int i=0; i<(signed long)(list->count()); i++)
		{
			cout << i << ": " << list->at(i)->startDate.toString() << "\n";
			text_xml+=addTagEventPast(list->at(i));
		}
	text_xml+="\t</data>\n";
	text_xml+="</PastEvents>";
	return text_xml;
}

/*!
    \fn FilePast::addTagEventPast(CalendarEventPast * temp)
	Method creates XML string describing past event
 	\param temp pointer to past event
	\return XML string describing past event
*/
QString FilePast::addTagEventPast(CalendarEventPast * temp)
{
	QString text_xml="\t\t<event>\n";
	
	text_xml+=addTagAny("date",3,temp->startDate.toString("yyyyMMdd"));
	text_xml+=addTagAny("text",3,temp->text);
	text_xml+=addTagAny("ttl",3,QString("%1").arg(temp->timeToLife));
	
	text_xml+="\t\t</event>\n";	
	
	return text_xml;
}






/*!
    \fn FilePast::createPastList(QString filename)
	Method creates global list of past events using xml configuration file.
 	\param filename name of file with past events stored in.
 */
QDate FilePast::createPastList(QString filename)
{
	QDate sD;
	QDomElement doc;
	QDate current=QDate::currentDate();
	if(openXMLFileForRead(filename, "PastEvents", &doc))
		return current.addDays(-365);
	QDomNode n = doc.firstChild();
    while( !n.isNull() ) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if( !e.isNull() ) {
			if (e.tagName()=="data")
				{
					QDomNode n1;
					n1=n.firstChild();
					retrievePastEvents(n1);
				}
			if (e.tagName()=="header")
				{
					QDomNode h1;
					h1=n.firstChild();
					while( !h1.isNull())
					{
						QDomElement e1=h1.toElement();
						if(!e1.isNull())
						{
							//cout << e1.tagName() << "\n";
							if(e1.tagName()=="SavingDate")
							{
								QString temp=e1.text();
								sD.setYMD(atoi(temp.left(4)), atoi(temp.mid(4,2)), atoi(temp.mid(6,2)));
								if(current.daysTo(sD)<-365)
									sD=current.addDays(-365);
							}
						}
						h1=h1.nextSibling();
					}
				}
        }
        n = n.nextSibling();
    }
	if (sD.isNull())
		sD=current.addDays(-365);
	return sD;
}


/*!
    \fn FilePast::retrievePastEvents(QDomNode n)
	Method retrieves all past events saved in past events cofniguration file
 	\param n pointer to first &lt;event&gt; tag.
 */
void FilePast::retrievePastEvents(QDomNode n)
{
 	//pobiera wszystkie eventy
	while (!n.isNull() )
	{
	QDomElement e = n.toElement();
	if( !e.isNull() )
		{
		if (e.tagName()=="event")
			{
				QDomNode n1;
				n1=n.firstChild();
				retrievePastEventParameters(n1);
			}
		}
	n=n.nextSibling();
	}
}


/*!
    \fn FilePast::retrievePastEventParameters(QDomNode n)
	Method retrieves data of past event from xml configuration file
	\param n pointer to first data of past event
 */
void FilePast::retrievePastEventParameters(QDomNode n)
{

	QString text, temp, sError="", errorCaption="";
	QDate dStart;
	QDate bigdate(2999,12,31);
	int ttl=0;
	int error=0;
	dStart=bigdate;
	
	while(!n.isNull() )
	{
		QDomElement e = n.toElement();
		if( !e.isNull() )
		{
			//cout << "\t\t" << e.tagName() << "\n";	
			temp=e.text();
			
			if(e.tagName()=="date")
			{
				if (temp.length()==8)
				{
					dStart.setYMD(atoi(temp.left(4)), atoi(temp.mid(4,2)), atoi(temp.mid(6,2)));
					//cout << "\t\t\t" << dStart.toString() << "\n";
				}
				else
				{
					error|=CalEnums::ERR_DSTART;
					sError+="date = "+temp+QObject::tr("\nShould be in YYYYMMDD format (ie. 20021231)\n");
				}
			}
			else if(e.tagName()=="text")
			{
				text=temp;
				//cout << "\t\t\t" << text << "\n";
				if(text.length()==0)
					{
					error|=CalEnums::ERR_GENERAL;
					sError+="text = "+temp+QObject::tr("\nShould be populated\n");
					}
			}
			else if(e.tagName()=="ttl")
			{
				ttl=atoi(temp);
				if(ttl<=0)
					{
						error|=CalEnums::ERR_GENERAL;
						sError+="ttl = "+temp+QObject::tr("\nShould be a number greater than 0 and less than 366\n");
					}
				else if(ttl>365)
					{
						error|=CalEnums::ERR_GENERAL;
						sError+="ttl = "+temp+QObject::tr("\nShould be less than 366. This value is automagicaly changed to 365\n");
						ttl=365;
					}
			}
		}
	n=n.nextSibling();
	}

	if(!(error&CalEnums::ERR_DSTART) && dStart==bigdate)
		{
			error|=CalEnums::ERR_DSTART;
			sError+=QObject::tr("Past event date has to be specified\n");
		}
	//sprawdzenie czy nie ma bdw
	if(error)
	{
		errorCaption+=QObject::tr("Check, which event has an error and add it again.\nYou can also edit manualy file with past events if you want\n(~/.tskal/past.xml).");
		errorCaption+="\n_____________________________________\n";
		errorCaption+=QObject::tr("Error concerned the following event:");
		if(!(error & CalEnums::ERR_DSTART))
			errorCaption+="\ndate="+dStart.toString();
		if(!(error & CalEnums::ERR_GENERAL))
			errorCaption+="\ntekst="+text;
		errorCaption+=QString("%1").arg(error);
		errorCaption+="\n_____________________________________\n";
		errorCaption+=QObject::tr("Errors occured:\n")+sError;
		QMessageBox a(QObject::tr("Error in XML file of past events"), errorCaption, QMessageBox::Warning,0,0,0);
		a.exec();
	}
	else
	{
		if(dStart.addDays(ttl)>QDate::currentDate())
		{
			CalendarEventPast * tempkPE=new CalendarEventPast(dStart, text, ttl);
			list->append(tempkPE);
		}
	}
}



