/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "fperiod.h"
#include <qobject.h>
#include <qmessagebox.h>
#include "consts.h"

/*!
	fn FilePeriodic::FilePeriodic(QPtrList<CalendarEventPeriodic> *l)
	Constructor of class
	\param l pointer to global list of periodic events
*/
FilePeriodic::FilePeriodic(QPtrList<CalendarEventPeriodic> *l)
{
	list=l;
}


/*!
	fn FilePeriodic::~FilePeriodic()
	Default destructor of a class
*/
FilePeriodic::~FilePeriodic()
{
}

/*!
    \fn FilePeriodic::addTagEventPeriodic(CalendarEventPeriodic* temp)
	Method creates a xml string describing periodic event
 	\param temp pointer to event
	\return XML String describing periodic event
*/
QString FilePeriodic::addTagEventPeriodic(CalendarEventPeriodic* temp) 
{
	QString text_xml="\t\t<event>\n";
	QDate start_date;
	QDate end_date;
	int day_period=-1;
	int month_period=-1;
	int year_period=-1;
	int week_period=-1;
	QString text;

	text=temp->text;
	start_date=temp->startDate;
	end_date = temp->endDate;
	
	switch(temp->eventType)
	{
		case CalEnums::DAILY:
			day_period=1;
			break;
		case CalEnums::MONTHLY:
			month_period=temp->period;
			break;
		case CalEnums::WEEKLY:
			week_period=temp->period;
			break;
		case CalEnums::YEARLY:
			year_period=temp->period;
			break;
		case CalEnums::PERIOD:
			day_period=temp->period;
			break;
		default:
			day_period=0;
	}				
	
	//<start_date>
	text_xml+=addTagAny("start_date",3,start_date.toString("yyyyMMdd"));
	//<end_date/>
	text_xml+=addTagAny("end_date",3,end_date.toString("yyyyMMdd"));
	
	//<weekday/>
	if(temp->eventType==CalEnums::DAYS)
		for(int i=0; i<7; i++)
		{
			if(temp->days[i])
				text_xml+=addTagAny("weekday", 3, QString("%1").arg(i+1));
		}
	else //fill xml without weekdays
		text_xml+=addTagAny("weekday",3,"");
	
	
	//<day_period/>
	if(day_period>0)
		text_xml+=addTagAny("day_period",3,QString("%1").arg(day_period));
	else
		text_xml+=addTagAny("day_period",3,"");
	
	//<week_period>
	if(week_period>0)
		text_xml+=addTagAny("week_period",3, QString("%1").arg(week_period));
	else
		text_xml+=addTagAny("week_period",3,"");
		
	//<month_period/>
	if(month_period>0)
		text_xml+=addTagAny("month_period",3,QString("%1").arg(month_period));
	else
		text_xml+=addTagAny("month_period",3,"");
		
	//<year_period/>
	if(year_period>0)
		text_xml+=addTagAny("year_period",3,QString("%1").arg(year_period));
	else
		text_xml+=addTagAny("year_period",3,"");
	
	text_xml+=addTagAny("autodelete", 3, temp->deleteFromFileAfterEnd?"1":"0");
	
	text_xml+=addTagAny("forgotafter", 3, QString("%1").arg(temp->maxDaysPassedForPeriodic));
		
	text_xml+=addTagAny("text",3,text);
	
	text_xml+="\t\t</event>\n";	
	
	return text_xml;
}


/*!
    \fn FilePeriodic::createPeriodicXML()
	Creates configuration xml of periodic events to save in xml file
	\return String with configuration of periodic events to save in a xml file
 */
QString FilePeriodic::createPeriodicXML()
{
	QString text_xml;
	//create doc with <PeriodicEvents>
	text_xml+="<PeriodicEvents>\n";
	//insert <head> section
	text_xml+="\t<head>\n";
		//insert <description>
	text_xml+="\t\t<description>";
	text_xml+=QObject::tr("File with periodic events");
	text_xml+="</description>\n";
	text_xml+="\t</head>\n";
	//insert <data> section
	text_xml+="\t<data>\n";
		//for each periodic event
		for(int i=0; i<(signed int)(list->count()); i++)
			text_xml+=addTagEventPeriodic(list->at(i));
	text_xml+="\t</data>\n";
	text_xml+="</PeriodicEvents>";
	return text_xml;
}


/*!
    \fn FilePeriodic::createPeriodicList(QString xmlfile)
	Creates a list of periodic events from xml file
 	\param xmlfile not used
 */
void FilePeriodic::createPeriodicList(QString xmlfile)
{
	QDomElement doc;
	if(openXMLFileForRead(xmlfile, "PeriodicEvents", &doc))
		return;
	QDomNode n = doc.firstChild();
    while( !n.isNull() ) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if( !e.isNull() ) {
			if (e.tagName()=="data")
				{
					QDomNode n1;
					n1=n.firstChild();
					retrievePeriodicEvents(n1);
				}
        }
        n = n.nextSibling();
    }
}


/*!
    \fn FilePeriodic::retrievePeriodicEvents(QDomNode n)
	Loads to periodic events list each event from xml file
 	\param n pointer to beginning of data section in xml file
 */
void FilePeriodic::retrievePeriodicEvents(QDomNode n)
{
 	//pobiera wszystkie eventy
	while (!n.isNull() )
	{
	QDomElement e = n.toElement();
	if( !e.isNull() )
		{
		if (e.tagName()=="event")
			{
				QDomNode n1;
				n1=n.firstChild();
				retrievePeriodicEventParameters(n1);
			}
		}
	n=n.nextSibling();
	}
}

/*!
    \fn FilePeriodic::retrievePeriodicEventParameters(QDomNode n)
	Retrieves parameters of periodic event from xml file.
 	\param n pointer to event section in xml file (event)
 */
void FilePeriodic::retrievePeriodicEventParameters(QDomNode n)
{
	//pobiera parametry eventw
	int error=0;
	QString sError;
	QString errorCaption;
	QDate dStart, dEnd;
	int dayPeriod=0;
	int weekPeriod=0;
	int monthPeriod=0;
	int yearPeriod=0;
	bool days[]={FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE};
	QString text="";;
	QString temp;
	int itemp;
	QDate bigdate;
	bigdate.setYMD(2999,12,31);
	dStart=bigdate;
	dEnd=bigdate;
	bool autodelete=TRUE;
	int forgotafter=0;
	
	while(!n.isNull() )
	{
		QDomElement e = n.toElement();
		if( !e.isNull() )
		{
			//cout << "\t\t" << e.tagName() << "\n";	
			temp=e.text();
			if(!temp.length())
				;
			else if(e.tagName()=="start_date")
			{
				if (temp.length()==8)
				{
					dStart.setYMD(atoi(temp.left(4)), atoi(temp.mid(4,2)), atoi(temp.mid(6,2)));
					//cout << "\t\t\t" << dStart.toString() << "\n";
				}
				else
				{
					error|=CalEnums::ERR_DSTART;
					sError+="start_date = "+temp+QObject::tr("\nshould be in YYYYMMDD format (ie. 20021231)\n");
				}
			}
			else if(e.tagName()=="end_date")
			{
				if(temp.length()==8)
				{
					dEnd.setYMD(atoi(temp.left(4)), atoi(temp.mid(4,2)), atoi(temp.mid(6,2)));
					//cout << "\t\t\t" << dEnd.toString() << "\n";
				}
				else
				{
					error|=CalEnums::ERR_DEND;
					sError+="end_date = "+temp+QObject::tr("\nshould be in YYYYMMDD format (ie. 20021231)\n");
				}
			}
			else if(e.tagName()=="weekday")
			{
				itemp=atoi(temp);
				if(itemp>7 || itemp<1)
				{
					error|=CalEnums::ERR_WEEKDAY;
					sError+="weekday = "+temp+QObject::tr("\nshould be from 1 for Monday to 7 for Sunday\n");
				}
				else
				{
					days[itemp-1]=TRUE;
					//cout << "\t\t\t" << itemp << "\n";
				}
			}
			else if(e.tagName()=="day_period")
			{
				itemp=atoi(temp);
				if(itemp<1)
					{
					error|=CalEnums::ERR_PDAY;
					sError+="day_period = "+temp+QObject::tr("\nShould be greater than or equal to 1\n");
					}
				else
				{
					dayPeriod=itemp;
					//cout << "\t\t\t" << dayPeriod << "\n";
				}
			}
			else if(e.tagName()=="month_period")
			{
				itemp=atoi(temp);
				if(itemp<1)
				{
					error|=CalEnums::ERR_PMONTH;
					sError+="month_period = "+temp+QObject::tr("\nShould be greater than or equal to 1\n");
				}
				else
				{
					monthPeriod=itemp;
				//cout << "\t\t\t" << monthPeriod << "\n";
				}
			}
			else if(e.tagName()=="week_period")
			{
				itemp=atoi(temp);
				if(itemp<1)
				{
					error|=CalEnums::ERR_PWEEK;
					sError+="week_period = "+temp+QObject::tr("\nShould be greater than or equal to 1\n");
				}
				else
				{
					weekPeriod=itemp;
				//cout << "\t\t\t" << weekPeriod << "\n";
				}
			}
			else if(e.tagName()=="year_period")
			{
				itemp=atoi(temp);
				if(itemp<1)
				{	
					error|=CalEnums::ERR_PYEAR;
					sError+="year_period = "+temp+QObject::tr("\nShould be greater than or equal to 1\n");
				}
				else
				{
					yearPeriod=itemp;
					//cout << "\t\t\t" << yearPeriod << "\n";
				}
			}
			else if(e.tagName()=="forgotafter")
			{
				 itemp=atoi(temp);
				 if(itemp<0)
				 	{
						error|=CalEnums::ERR_GENERAL;
						sError+="forgotafter = "+temp+QObject::tr("\nShould be greater or equal to 0\n");
					}
				 else
				 	{
					forgotafter=itemp;
					}
			}
			else if(e.tagName()=="autodelete")
			{
				autodelete=(temp=="1"?TRUE:FALSE);
			}
			else if(e.tagName()=="text")
			{
				text=temp;
				//cout << "\t\t\t" << text << "\n";
			}
		}
	n=n.nextSibling();
	}
	
	
	//data processing, error finding
	if(!error)
	{
		if(dStart==bigdate)
		{
			error|=CalEnums::ERR_GENERAL;
			sError+=QObject::tr("Lack of beginning date\n");
		}
		if(text.length()==0)
		{
			error|=CalEnums::ERR_GENERAL;
			sError+=QObject::tr("Lack of event description\n");
		}
		if(!(days[0] || days[1] || days[2] || days[3] || days[4] || days[5] || days[6] || dayPeriod || monthPeriod || weekPeriod || yearPeriod))
		{
			error|=CalEnums::ERR_GENERAL;
			sError+=QObject::tr("There is no rules of event repeating\n");
		}
		else if(((days[0] || days[1] || days[2] || days[3] || days[4] || days[5] || days[6])+ (dayPeriod!=0) + (weekPeriod!=0) + (monthPeriod!=0) + (yearPeriod!=0))>1)
		{
			error|=CalEnums::ERR_GENERAL;
			sError+=QObject::tr("There are more than one rule of event repetition\n");
		}
	}
	if(error)
	{
		errorCaption+=QObject::tr("Check, which event has an error and add it again. \nYou can also edit manualy file with periodic events\n(~/.tskal/periodic.xml).");
		errorCaption+="\n_____________________________________\n";
		errorCaption+=QObject::tr("Error concerned to following event:");
		if(!(error&CalEnums::ERR_DSTART) && dStart!=bigdate)
			errorCaption+="\nstart_date="+dStart.toString();
		if(!(error&CalEnums::ERR_DEND) && dEnd!=bigdate)
			errorCaption+="\nend_date="+dEnd.toString();
		if(!(error&CalEnums::ERR_WEEKDAY) && (days[0] || days[1] || days[2] || days[3] || days[4] || days[5] || days[6]))
			{
				errorCaption+="\nweekday=";
					for (int i =0; i<7; i++)
						if(days[i])
							errorCaption+=QString("%1").arg(i+1)+",";
			}
		if(!(error&CalEnums::ERR_PDAY) && dayPeriod)
			errorCaption+="\nday_period="+QString("%1").arg(dayPeriod);
		if(!(error&CalEnums::ERR_PWEEK) && weekPeriod)
			errorCaption+="\nweek_period="+QString("%1").arg(weekPeriod);
		if(!(error&CalEnums::ERR_PMONTH) && monthPeriod)
			errorCaption+="\nmonth_period="+QString("%1").arg(monthPeriod);
		if(!(error&CalEnums::ERR_PYEAR) && yearPeriod)
			errorCaption+="\nyear_period="+QString("%1").arg(yearPeriod);
		errorCaption+="\ntext = " + text;
		errorCaption+="\n_____________________________________\n";
		errorCaption+=QObject::tr("Errors occured:\n")+sError;
		QMessageBox a(QObject::tr("Error in XML file of periodic events"), errorCaption, QMessageBox::Warning,0,0,0);
		a.exec();
	}
	
	
	
	if(!error)
	//event is added only there is no error during processing
	{
	CalEnums::event_type type=CalEnums::DAYS;
	int period=0;
	bool *weekdays=NULL;
	if(dayPeriod)
		{
		if(dayPeriod==1)
			{
			type=CalEnums::DAILY;
			period=1;
			}
		else
			{
			type=CalEnums::PERIOD;
			period=dayPeriod;
			}
		}
	else if(weekPeriod)
		{
			type=CalEnums::WEEKLY;
			period=weekPeriod;
		}
	else if(monthPeriod)
		{
			type=CalEnums::MONTHLY;
			period=monthPeriod;
		}
	else if(yearPeriod) 
		{
			type=CalEnums::YEARLY;
			period=yearPeriod;
		}
	else
		{
		weekdays=days;
		}
	CalendarEventPeriodic * tempkPE=new CalendarEventPeriodic(text, dStart, dEnd, type, period, weekdays, autodelete, forgotafter);
	list->append(tempkPE);
	}	
}


