/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "theme.h"
#include <qnamespace.h>
#include <qsettings.h>
#include <qdir.h>


/*!
    \fn CalendarTheme::CalendarTheme()
	Default constructor
 */
CalendarTheme::CalendarTheme()
{
	name="";
}


/*!
    \fn CalendarTheme::~CalendarTheme
	Default destructor
 */
CalendarTheme::~CalendarTheme()
{
}

/*!
    \fn CalendarTheme::setTheme(int HEIGHT, QString path, QString themename)
	Loads theme from file, calculates font sizes
	\param HEIGHT height of calendar card. Used for calculation of font size
	\param path path to theme directory
	\param themename name of theme or the whole path to theme directory (placed in non-standard directory)
 */
void CalendarTheme::setTheme(int HEIGHT, QString path, QString themename)
{
   	QSettings * th=new QSettings(QSettings::Ini);
	th->setPath("mike.oldfield.org.pl/tytus", "Kalendarz", QSettings::User);

	
	{ //choose the right themepath
		QDir directory(path+themename);
		if (directory.exists())
			th->insertSearchPath(QSettings::Unix,path+themename);
		else 
			th->insertSearchPath(QSettings::Unix,themename);
	}

	QString sFC = th->readEntry("/theme/NormalDayFontColor", "blue");
	QString sF = th->readEntry("/theme/NormalDayFont", "Arial");
	QString sAFC = th->readEntry("/theme/ActiveColor", "yellow");
	QString sAF = th->readEntry("/theme/ActiveFont","");
	QString sSFC = th->readEntry("/theme/SundayDayFontColor", "red");
	QString sSF = th->readEntry("/theme/SundayDayFont", "");
	QString sMFC = th->readEntry("/theme/MonthColor", "black");
	QString sMF = th->readEntry("/theme/MonthFont", "");
	QString sBC = th->readEntry("/theme/BackgroundColor", "gray");

	
	//normal cards
	cardFontColor=QColor(sFC);
	if (!cardFontColor.isValid()) cardFontColor=Qt::blue;
	
	cardFont=QFont(sF);
	cardFont.setPixelSize( HEIGHT/3 );
	
	//active cards
	activeCardFontColor=QColor(sAFC);
	if(!activeCardFontColor.isValid()) activeCardFontColor=Qt::yellow;
	
	if (sAF.isEmpty()) sAF=sF;
	activeCardFont=QFont(sAF);
	activeCardFont.setPixelSize( HEIGHT/3 );
	
	//sunday cards
	sundayFontColor=QColor(sSFC);
	if (!sundayFontColor.isValid()) sundayFontColor=Qt::red;
	
	if(sSF.isEmpty())  sSF=sF;
	sundayFont=QFont(sSF);
	sundayFont.setPixelSize( HEIGHT/3 );

	//months display
	monthFontColor=QColor(sMFC);
	if(!monthFontColor.isValid()) monthFontColor=Qt::black;
	if(sMF.isEmpty()) sMF = sF;
	monthFont=QFont(sMF);
	monthFont.setPixelSize( (HEIGHT>5)?(HEIGHT/3-1):(1) );

	//background color of application
	backgroundApplicationColor=QColor(sBC);
	if (!backgroundApplicationColor.isValid()) backgroundApplicationColor=Qt::gray;
				
	delete th;
}
